@@ -1,10 +1,11 @@
-use syn;
-use syn::*;
-use quote::Tokens;
+use std::str::FromStr;
+
+use proc_macro2::{Span, TokenStream};
+use quote::ToTokens;
+use syn::{self, Data, DeriveInput, Field, Fields, Ident, Meta, NestedMeta};
 
 use util::*;
 
-
 #[derive(Default, Debug)]
 struct ObjAttrs {
     name: Option<String>,
@@ -16,25 +17,35 @@ impl ObjAttrs {
     fn from_input(input: &DeriveInput) -> ObjAttrs {
         let mut res = ObjAttrs::default();
 
+        // Check doc comments for description.
+        res.description = get_doc_comment(&input.attrs);
+
         // Check attributes for name and description.
-        if let Some(items) = get_graphl_attr(&input.attrs) {
+        if let Some(items) = get_graphql_attr(&input.attrs) {
             for item in items {
-                if let Some(val) = keyed_item_value(item, "name", true) {
-                    res.name = Some(val);
-                    continue;
+                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "name", AttributeValidation::String)  {
+                    if is_valid_name(&*val) {
+                        res.name = Some(val);
+                        continue;
+                    } else {
+                        panic!(
+                            "Names must match /^[_a-zA-Z][_a-zA-Z0-9]*$/ but \"{}\" does not",
+                            &*val
+                        );
+                    }
                 }
-                if let Some(val) = keyed_item_value(item, "description", true) {
+                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "description", AttributeValidation::String)  {
                     res.description = Some(val);
                     continue;
                 }
                 match item {
-                    &NestedMetaItem::MetaItem(MetaItem::Word(ref ident)) => {
+                    NestedMeta::Meta(Meta::Word(ref ident)) => {
                         if ident == "_internal" {
                             res.internal = true;
                             continue;
                         }
-                    },
-                    _ => {},
+                    }
+                    _ => {}
                 }
                 panic!(format!(
                     "Unknown attribute for #[derive(GraphQLInputObject)]: {:?}",
@@ -58,29 +69,39 @@ impl ObjFieldAttrs {
     fn from_input(variant: &Field) -> ObjFieldAttrs {
         let mut res = ObjFieldAttrs::default();
 
+        // Check doc comments for description.
+        res.description = get_doc_comment(&variant.attrs);
+
         // Check attributes for name and description.
-        if let Some(items) = get_graphl_attr(&variant.attrs) {
+        if let Some(items) = get_graphql_attr(&variant.attrs) {
             for item in items {
-                if let Some(val) = keyed_item_value(item, "name", true) {
-                    res.name = Some(val);
-                    continue;
+                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "name", AttributeValidation::String)  {
+                    if is_valid_name(&*val) {
+                        res.name = Some(val);
+                        continue;
+                    } else {
+                        panic!(
+                            "Names must match /^[_a-zA-Z][_a-zA-Z0-9]*$/ but \"{}\" does not",
+                            &*val
+                        );
+                    }
                 }
-                if let Some(val) = keyed_item_value(item, "description", true) {
+                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "description", AttributeValidation::String)  {
                     res.description = Some(val);
                     continue;
                 }
-                if let Some(val) = keyed_item_value(item, "default", true) {
+                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "default", AttributeValidation::Any) {
                     res.default_expr = Some(val);
                     continue;
                 }
                 match item {
-                    &NestedMetaItem::MetaItem(MetaItem::Word(ref ident)) => {
+                    NestedMeta::Meta(Meta::Word(ref ident)) => {
                         if ident == "default" {
                             res.default = true;
                             continue;
                         }
                     }
-                    _ => {},
+                    _ => {}
                 }
                 panic!(format!(
                     "Unknown attribute for #[derive(GraphQLInputObject)]: {:?}",
@@ -92,17 +113,17 @@ impl ObjFieldAttrs {
     }
 }
 
-pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
-    let fields = match ast.body {
-        Body::Struct(ref data) => match data {
-            &VariantData::Struct(ref fields) => fields,
+pub fn impl_input_object(ast: &syn::DeriveInput) -> TokenStream {
+    let fields = match ast.data {
+        Data::Struct(ref data) => match data.fields {
+            Fields::Named(ref named) => named.named.iter().collect::<Vec<_>>(),
             _ => {
                 panic!(
                     "#[derive(GraphQLInputObject)] may only be used on regular structs with fields"
                 );
             }
         },
-        Body::Enum(_) => {
+        _ => {
             panic!("#[derive(GraphlQLInputObject)] may only be applied to structs, not to enums");
         }
     };
@@ -111,15 +132,16 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
     let ident = &ast.ident;
     let attrs = ObjAttrs::from_input(ast);
     let name = attrs.name.unwrap_or(ast.ident.to_string());
+    let generics = &ast.generics;
 
     let meta_description = match attrs.description {
         Some(descr) => quote!{ let meta = meta.description(#descr); },
         None => quote!{ let meta = meta; },
     };
 
-    let mut meta_fields = Vec::<Tokens>::new();
-    let mut from_inputs = Vec::<Tokens>::new();
-    let mut to_inputs = Vec::<Tokens>::new();
+    let mut meta_fields = TokenStream::new();
+    let mut from_inputs = TokenStream::new();
+    let mut to_inputs = TokenStream::new();
 
     for field in fields {
         let field_ty = &field.ty;
@@ -134,21 +156,30 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
             }
             None => {
                 // Note: auto camel casing when no custom name specified.
-                ::util::to_camel_case(field_ident.as_ref())
+                ::util::to_camel_case(&field_ident.to_string())
             }
         };
         let field_description = match field_attrs.description {
             Some(s) => quote!{ let field = field.description(#s); },
-            None => quote!{ let field = field; },
+            None => quote!{},
         };
 
         let default = {
             if field_attrs.default {
-                Some(quote! { Default::default() } )
+                Some(quote! { Default::default() })
             } else {
                 match field_attrs.default_expr {
-                    Some(ref def) => match syn::parse_token_trees(def) {
-                        Ok(t) => Some(quote! { #(#t)* }),
+                    Some(ref def) => match ::proc_macro::TokenStream::from_str(def) {
+                        Ok(t) => match syn::parse::<syn::Expr>(t) {
+                            Ok(e) => {
+                                let mut tokens = TokenStream::new();
+                                e.to_tokens(&mut tokens);
+                                Some(tokens)
+                            }
+                            Err(_) => {
+                                panic!("#graphql(default = ?) must be a valid Rust expression inside a string");
+                            }
+                        },
                         Err(_) => {
                             panic!("#graphql(default = ?) must be a valid Rust expression inside a string");
                         }
@@ -170,16 +201,15 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
                 }
             }
         };
-        let meta_field = quote!{
+        meta_fields.extend(quote!{
             {
                 #create_meta_field
                 #field_description
                 field
             },
-        };
-        meta_fields.push(meta_field);
+        });
 
-        // Buil from_input clause.
+        // Build from_input clause.
 
         let from_input_default = match default {
             Some(ref def) => {
@@ -190,7 +220,7 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
             None => quote!{},
         };
 
-        let from_input = quote!{
+        from_inputs.extend(quote!{
             #field_ident: {
                 // TODO: investigate the unwraps here, they seem dangerous!
                 match obj.get(#name) {
@@ -202,18 +232,16 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
                     },
                 }
             },
-        };
-        from_inputs.push(from_input);
+        });
 
         // Build to_input clause.
-        let to_input = quote!{
+        to_inputs.extend(quote!{
             (#name, self.#field_ident.to_input_value()),
-        };
-        to_inputs.push(to_input);
+        });
     }
 
     let body = quote! {
-        impl _juniper::GraphQLType for #ident {
+        impl #generics _juniper::GraphQLType for #ident #generics {
             type Context = ();
             type TypeInfo = ();
 
@@ -221,7 +249,10 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
                 Some(#name)
             }
 
-            fn meta<'r>(_: &(), registry: &mut _juniper::Registry<'r>) -> _juniper::meta::MetaType<'r> {
+            fn meta<'r>(
+                _: &(),
+                registry: &mut _juniper::Registry<'r>
+            ) -> _juniper::meta::MetaType<'r> {
                 let fields = &[
                     #(#meta_fields)*
                 ];
@@ -231,8 +262,8 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
             }
         }
 
-        impl _juniper::FromInputValue for #ident {
-            fn from_input_value(value: &_juniper::InputValue) -> Option<#ident> {
+        impl #generics _juniper::FromInputValue for #ident #generics {
+            fn from_input_value(value: &_juniper::InputValue) -> Option<#ident #generics> {
                 if let Some(obj) = value.to_object_value() {
                     let item = #ident {
                         #(#from_inputs)*
@@ -245,7 +276,7 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
             }
         }
 
-        impl _juniper::ToInputValue for #ident {
+        impl #generics _juniper::ToInputValue for #ident #generics {
             fn to_input_value(&self) -> _juniper::InputValue {
                 _juniper::InputValue::object(vec![
                     #(#to_inputs)*
@@ -254,7 +285,10 @@ pub fn impl_input_object(ast: &syn::DeriveInput) -> Tokens {
         }
     };
 
-    let dummy_const = Ident::new(format!("_IMPL_GRAPHQLINPUTOBJECT_FOR_{}", ident));
+    let dummy_const = Ident::new(
+        &format!("_IMPL_GRAPHQLINPUTOBJECT_FOR_{}", ident),
+        Span::call_site(),
+    );
 
     // This ugly hack makes it possible to use the derive inside juniper itself.
     // FIXME: Figure out a better way to do this!
