@@ -7,37 +7,38 @@
 #![recursion_limit = "1024"]
 
 extern crate proc_macro;
-extern crate syn;
+extern crate proc_macro2;
 #[macro_use]
 extern crate quote;
+extern crate syn;
+#[macro_use]
+extern crate lazy_static;
+extern crate regex;
 
-mod util;
 mod derive_enum;
 mod derive_input_object;
 mod derive_object;
+mod util;
 
 use proc_macro::TokenStream;
 
 #[proc_macro_derive(GraphQLEnum, attributes(graphql))]
 pub fn derive_enum(input: TokenStream) -> TokenStream {
-    let s = input.to_string();
-    let ast = syn::parse_derive_input(&s).unwrap();
+    let ast = syn::parse::<syn::DeriveInput>(input).unwrap();
     let gen = derive_enum::impl_enum(&ast);
-    gen.parse().unwrap()
+    gen.into()
 }
 
 #[proc_macro_derive(GraphQLInputObject, attributes(graphql))]
 pub fn derive_input_object(input: TokenStream) -> TokenStream {
-    let s = input.to_string();
-    let ast = syn::parse_derive_input(&s).unwrap();
+    let ast = syn::parse::<syn::DeriveInput>(input).unwrap();
     let gen = derive_input_object::impl_input_object(&ast);
-    gen.parse().unwrap()
+    gen.into()
 }
 
 #[proc_macro_derive(GraphQLObject, attributes(graphql))]
 pub fn derive_object(input: TokenStream) -> TokenStream {
-    let s = input.to_string();
-    let ast = syn::parse_derive_input(&s).unwrap();
+    let ast = syn::parse::<syn::DeriveInput>(input).unwrap();
     let gen = derive_object::impl_object(&ast);
-    gen.parse().unwrap()
+    gen.into()
 }
